/*
 =============================================================
 PROJECT : FULTRA
 ---------------------------------------------------------
 Copyright (c) 2016 - Relution Developments Ltd
 Licensed under the MIT license.
 @author BluJay - JayDemtiri <jay.demitri.16@gmail.com>
 =============================================================
 NOTES:
 - All Gulp packages are appended with g
 - All Node pacakges are written in camelCase from the original
 - All Tasks are grouped into the function they serve with one
    one main task relying on several subtasks
 - Setting are kept in arrays relating to the object they are used for
 =============================================================
 Release History:
 v1.0 - Gulpfile finalized and released as of fultra v1
 */

"use strict";

//  Libs & Configs
var path = require('path');
var pkg = require('./package.json');
var gulp = require('gulp');
var gUtil = require('gulp-util');
var gProcessHtml = require('gulp-processhtml');
var gSass = require('gulp-sass');
var gSourceMaps = require('gulp-sourcemaps');
var gConcat = require('gulp-concat');
var browserSync = require('browser-sync').create();
var taskListing = require('gulp-task-listing');
var router = require('front-router');
var gHtml2Js = require('gulp-ng-html2js');
var gAutoPrefixer = require('gulp-autoprefixer');
var runSequence = require('run-sequence');

var gProcessHtmlSettings = {
    "process": true,
    "environment": (gUtil.env.env ? gUtil.env.env : "dist"),
    "data": {
        "title": "Documentation",
        "message": "This is Dist environment",
        "comment": "These comments will all be removed in the release environment"
    }
};
var browserSyncSettings = {
    server: {
        baseDir: "./docs/release",
        directory: false,
        index: "./index.html",
        //https: true,
        ghostMode: {
            clicks: true,
            forms: true,
            scroll: false
        }
    },
    logLevel: (gUtil.env.debug ? gUtil.env.debug : "silent"), //debug, info or silent
    logPrefix: "Host",
    logConnections: true,
    logFileChanges: true,
    notify: true,
    tunnel: false, //can replace with name
    online: true, //Will assume we are offline no tunnel,
    open: true // open, false, local, external, ui, tunnel
};

// General Tasks
gulp.task('clean', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('default', ['help'], function(){
    var notify = require('gulp-notify');
    return notify("Hello Gulp!");
});
gulp.task('info', function(){
    gUtil.log(gUtil.colors.cyan('Project Fultra'));
    gUtil.log(gUtil.colors.cyan('----------------------------------------------'));
    gUtil.log(gUtil.colors.cyan('Version: '), pkg.version);
    gUtil.log(gUtil.colors.cyan('----------------------------------------------'));
});
gulp.task('help', taskListing);

// Launch Tasks
gulp.task('launch', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('launch:docs', ['build'], function () {
    
    // When the server is running let the user know
    browserSync.emitter.on("init", function () {
        gUtil.log("Browsersync is running!");
    });
    
    // Initiate the server
    browserSync.init(browserSyncSettings);

    // Run watch tasks to watch elements and run corresponding build task
    
    //Watch scss
    gulp.watch(['./docs/src/assets/scss/*.scss','./libs/src/scss/**'], ['build:css']);
    
    gulp.watch(['./libs/vendor/foundation-apps/js/angular/components/**/*.html'], ['build:templateCache', 'build:js']);
    
    gulp.watch(['./docs/src/*.html','./docs/src/**/*.html'], ['build:html', 'build:js']);
    
    gulp.watch(['./docs/src/assets/js/*.js','./libs/src/js/*.js'],['build:js']);
    
    gulp.watch('./docs/release/**').on('change', browserSync.reload);
});

// Build Tasks
gulp.task('build', function(callback){
    runSequence('build:html','build:css','build:fonts','build:templateCache','build:js',callback)
});
gulp.task('build:html', function () {
    return gulp.src(
        [
            './docs/src/*.html',
            './docs/src/**/*.html'
        ]
    )
        .pipe(router({
            path: './docs/src/assets/js/routes.js',
            root: './docs/src/'
        }))
        .pipe(gProcessHtml(gProcessHtmlSettings))
        .pipe(gulp.dest('./docs/release'));
});
gulp.task('build:css', function(){
    return gulp.src('./docs/src/assets/scss/*.scss')
        .pipe(gSourceMaps.init())
        .pipe(gSass({
            includePaths: [
                './libs/src/scss/',
                './libs/vendor/'
            ],
            outputStyle: "nested",
            errLogToConsole: true
        }).on('error', gSass.logError))
        .pipe(gAutoPrefixer({
            browsers: ['last 2 versions'],
            cascade: false
        }))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./docs/release/assets/css'));
});
gulp.task('build:settings',function(cb){
//    var octophant = require('octophant');
//
//    return octophant(['./libs/src/scss/', './libs/vendor/foundation-apps/scss/'], {
//                title: 'Fultra for Sites Settings',
//                output: './libs/src/scss/_settings.scss',
//                groups: {
//                        'grid': 'The Grid',
//                        'off-canvas': 'Off-canvas',
//                        'typography-base': 'Base Typography'
//                      },
//                      sort: [
//                        'global',
//                        'breakpoints',
//                        'grid',
//                        'typography-base',
//                        'typography-helpers'
//                      ],
//                      imports: ['util/util'],
//                      _foundationShim: true
//              }, cb());
});
gulp.task('build:templateCache', function(){
    return gulp.src('./libs/vendor/foundation-apps/js/angular/components/**/*.html')
        .pipe(gHtml2Js({
            moduleName: "foundation.templates",
            prefix: "components/"
        }))
        .pipe(gConcat('foundation.templates.js'))
        .pipe(gulp.dest('./docs/src/assets/js'));
});
gulp.task('build:js', function(){
    return gulp.src(
        [
           // './libs/vendor/jquery/dist/jquery.js',
           // Angular and deps
            './libs/vendor/angular/angular.js',
            './libs/vendor/angular-animate/angular-animate.js',
            './libs/vendor/angular-ui-router/release/angular-ui-router.js',
            // Any Jquery elements
            './libs/vendor/fastclick/lib/fastclick.js',
            './libs/vendor/hammerjs/hammer.js',
            './libs/vendor/tether/dist/js/tether.js',
            // Foundation Apps Components & Services
            './libs/vendor/foundation-apps/js/angular/services/*.js',
            './libs/vendor/foundation-apps/js/angular/components/**/*.js',
            './libs/vendor/foundation-apps/js/angular/vendor/**/*.js',
            './libs/vendor/foundation-apps/js/vendor/**/*.js',
            // Bring it all together with the fultra namespace
            './libs/src/js/fultra.js',
            // All of the foundation html templates into a js cache module
            './docs/src/assets/js/foundation.templates.js',
            // Our generated routes
            './docs/src/assets/js/routes.js',
            // And finally our application specific scripts
            './docs/src/assets/js/app.js'
        ]
    )
        .pipe(gSourceMaps.init())
        .pipe(gConcat('app.js'))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./docs/release/assets/js/'))
});
gulp.task('build:fonts', function(){
    return gulp.src('./libs/vendor/font-awesome/fonts/**')
        .pipe(gulp.dest('./docs/release/assets/fonts/font-awesome/'))
});

gulp.task('release', function(callback){
    runSequence('release:css','release:fonts','release:templateCache','release:js',callback)
});
gulp.task('release:css', function(){
    return gulp.src('./docs/src/assets/scss/*.scss')
        .pipe(gSourceMaps.init())
        .pipe(gSass({
            includePaths: [
                './libs/src/scss/',
                './libs/vendor/'
            ],
            outputStyle: "nested",
            errLogToConsole: true
        }).on('error', gSass.logError))
        .pipe(gAutoPrefixer({
            browsers: ['last 2 versions'],
            cascade: false
        }))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./libs/release/css'));
});
gulp.task('release:templateCache', function(){
    return gulp.src('./libs/vendor/foundation-apps/js/angular/components/**/*.html')
        .pipe(gHtml2Js({
            moduleName: "foundation.templates",
            prefix: "components/"
        }))
        .pipe(gConcat('foundation.templates.js'))
        .pipe(gulp.dest('./libs/release/js'));
});
gulp.task('release:js', function(){
    return gulp.src(
        [
            // './libs/vendor/jquery/dist/jquery.js',
            // Angular and deps
            './libs/vendor/angular/angular.js',
            './libs/vendor/angular-animate/angular-animate.js',
            './libs/vendor/angular-ui-router/release/angular-ui-router.js',
            // Any Jquery elements
            './libs/vendor/fastclick/lib/fastclick.js',
            './libs/vendor/hammerjs/hammer.js',
            './libs/vendor/tether/dist/js/tether.js',
            // Foundation Apps Components & Services
            './libs/vendor/foundation-apps/js/angular/services/*.js',
            './libs/vendor/foundation-apps/js/angular/components/**/*.js',
            './libs/vendor/foundation-apps/js/angular/vendor/**/*.js',
            './libs/vendor/foundation-apps/js/vendor/**/*.js',
            // Bring it all together with the fultra namespace
            './libs/src/js/fultra.js'
        ]
        )
        .pipe(gSourceMaps.init())
        .pipe(gConcat('fultra.js'))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./libs/release/js/'))
});
gulp.task('release:fonts', function(){
    return gulp.src('./libs/vendor/font-awesome/fonts/**')
        .pipe(gulp.dest('./libs/release/fonts/font-awesome/'))
});

// Lint Tasks
gulp.task('lint', function(callback){
    runSequence('lint:scss','lint:css','lint:js',callback);
});
gulp.task('lint:scss', function(){
    var gScssLint = require('gulp-scss-lint');

   return gulp.src([
       './libs/src/scss/*.scss',
       './docs/src/assets/scss/*.scss'
   ])
        .pipe(gScssLint());
});
gulp.task('lint:css', function(){
    var gCssLint = require('gulp-csslint');
    return gulp.src('./docs/release/assets/css/*.css')
        .pipe(gCssLint())
        .pipe(gCssLint.formatter(require('csslint-stylish')))
});
gulp.task('lint:js', function(){
    var gJsHint = require('gulp-jshint');
    return gulp.src([
        './libs/src/js/*.js',
        './docs/src/assets/js/app.js'
    ])
        .pipe(gJsHint())
        .pipe(gJsHint.reporter('jshint-stylish'))
        //.pipe(gJsHint.reporter('jshint-stylish', {beep: true}))
       // .pipe(gJsHint.reporter('fail'))
});

// Testing Tasks
gulp.task('test', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('test:css', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('test:js', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});

